package net.sf.nebulacards;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;

/**
 * A simple applet to allow the launching of a client interface from a webpage.
 * It will pop up a window running the client specified in the <i>client</i>
 * parameter.  The host is determined strictly from the URL of our page.
 */
public class LauncherApplet
	extends Applet
	implements ActionListener
{
	/**
	 * The name of the referring host.  It is assumed that the game server is
	 * there because we can't connect anywhere else.
	 */
	String hostString;
	/**
	 * The class name of the user interface to be loaded by the client.
	 */
	String ifString;
	/**
	 * For displaying error messages.
	 */
	Label errorL;
	/**
	 * For entering the server's port.
	 */
	TextField portTF;
	/**
	 * For entering the player's name.
	 */
	TextField nameTF;
	/**
	 * The launch button.
	 */
	Button launchB;
	
	/**
	 * Get the name of the referring host from the document URL.
	 * Sets the <i>host</i> variable.
	 * @see host
	 */
	void determineHost()
	{
		hostString = getDocumentBase().getHost();
	}

	/**
	 * Determine which interface is to be loaded.
	 * Reads the value of the <i>interface</i> parameter in the HTML file.
	 * Defaults to net.sf.nebulacards.ui.gui.
	 * Sets the ifString variable.
	 */
	void determineInterface()
	{
		ifString = getParameter( "interface" );
		if (ifString == null)
			ifString = "net.sf.nebulacards.ui.gui";
	}

	/**
	 * Layout the components in the applet's display area.
	 */
	void initComponents()
	{
		/* Create components. */
		launchB = new Button( "Play Now" );
		portTF = new TextField( "8802", 5 );
		nameTF = new TextField( 20 );
		errorL = new Label( "" );

		/* Layout the display area. */
		setLayout( new GridLayout(5,2) );
		add( new Label("Host: ") );
		add( new Label(this.hostString) );
		add( new Label("Port: ") );
		add( portTF );
		add( new Label("Your Name: ") );
		add( nameTF );
		add( new Label("Launch: ") );
		add( launchB );
		add( new Label("Message: ") );
		add( errorL );

		/* Listen to launch button. */
		launchB.addActionListener( this );
	}

	public void init()
	{
		determineHost();
		determineInterface();
		initComponents();
	}

	public void actionPerformed( ActionEvent e )
	{
		if (e.getSource() == launchB)
		{
			/* construct argument list for call to client */
			String[] parms = { "-p", portTF.getText(), "-h", hostString,
							"-i", ifString, "-n", nameTF.getText() };
			/* check for valid parameters */
			if (parms[1].length() < 1)
			{
				errorL.setText( "You must specify a port." );
			}
			else if (parms[7].length() < 1)
			{
				errorL.setText( "You must provide your name." );
			}
			else
			{
				errorL.setText( "Loading client..." );
				client c = new client( parms );
				errorL.setText( "Running client..." );
				c.run();
			}
		}
	}
}
