package net.sf.nebulacards;

import java.net.*;
import java.io.*;
import net.sf.nebulacards.main.*;
import net.sf.nebulacards.ui.*;
import net.sf.nebulacards.netpkt.*;
import java.lang.reflect.*;

/**
 * Client class for Nebula Cards.
 * @author James Ranson
 */
class client
    extends Thread
	implements PacketListener
{
    public static final boolean verbose = false;
    String[] args;
    Socket syncSock;
	CommThread com;
    boolean inGame = false;
	Queue packets;

    public static void main( String[] args )
        throws Exception
    {
        client c = new client( args );
        c.run();
        c = null;
        System.exit( 0 );
    }

    public client( String[] cmdline )
	{
		args = (String[])cmdline.clone();
		packets = new Queue();
	}
    
    public void run()
    {
        BufferedReader in = new BufferedReader( new InputStreamReader(System.in) );
        String pname = null, portString = null, ifString = null, hostString = null;

        // parse command line
        for (int i = 0; i < args.length; i++) {
            if (args[i].equals( "-p" )) {
                try { portString = args[++i]; }
                catch (Exception e) { portString = null; }
                continue;
            }
            if (args[i].equals( "-h" )) {
                try { hostString = args[++i]; }
                catch (Exception e) { hostString = null; }
                continue;
            }
            if (args[i].equals( "-n" )) {
                try { pname = args[++i]; }
                catch (Exception e) { pname = null; }
                continue;
            }
            if (args[i].equals( "-i" )) {
                try { ifString = args[++i]; }
                catch (Exception e) { ifString = null; }
                continue;
            }
        }
        
        // get player's name
        if (pname == null) {
            System.err.print( "Player's Name: " );
            System.err.flush();
            try { pname = in.readLine(); }
            catch (IOException e) {}
        }
        if (verbose) System.out.println( "Hello, " + pname + "." );
        Thread.yield();

        // get interface
        if (ifString == null) {
            System.err.println( "\nYou have to choose an interface.  Some options are:" );
            System.err.println( "net.sf.nebulacards.ui.SpadesComp    -- Spades computer player" );
            System.err.println( "net.sf.nebulacards.ui.gui           -- human player" );
            System.err.print( "\nInterface: " );
            try { ifString = in.readLine(); }
            catch (IOException e) {}
        }
        Class UI = null;
        try { UI = Class.forName( ifString ); }
        catch ( ClassNotFoundException e ) {
            System.err.println( "Error:  Cannot open interface: " + args[1] );
            try { Thread.sleep( 3000 ); }
            catch (InterruptedException f) {}
            return;
        }

        if (hostString == null) {
            System.err.println( "\nPlease enter the host name or IP address of the server that will" );
            System.err.println( "host this game.  (If the server is on the same machine as this client," );
            System.err.print( "then leave this blank.)\n\nServer: " );
            System.err.flush();
            try { hostString = in.readLine(); }
            catch (Exception e) {}
            if (hostString.equals("")) hostString = "127.0.0.1";
        }
        if (portString == null) {
            System.err.print( "Server's listening port: " );
            System.err.flush();
            try { portString = in.readLine(); }
            catch (Exception e) {}
        }
        int port = Integer.parseInt( portString );
        try
		{
			syncSock = new Socket( hostString, port );
        	if (verbose)
	            System.out.println( "Connected on port " + syncSock.getPort() );
			com = new CommThread( syncSock, 0 );
			com.addPacketListener( this );
			com.start();
		}
        catch (Exception e) {
            System.err.println( "Fatal Error while connecting to server:" );
            System.err.println( e.getMessage() );
            return;
        }

		int where = -1;  // where the new player will be seated
		try
		{
			com.send( new JoinPkt(pname) );
            // get response
			while (packets.empty())
				block( 1000 );
			Object o = packets.dequeue();
            if (o instanceof AcceptPkt) {
                if (verbose) System.out.println( "Accepted" );
                inGame = true;
				where = ((AcceptPkt)o).getParameter();
            }
            else {
                if (verbose)
				{
					System.out.println( "Rejected by receiving the following packet: " );
					System.out.println( o );
				}
                return;
            }
        }
        catch (Exception e) {
            System.err.println( "Fatal Error in client: " + e.getMessage() );
			e.printStackTrace( System.err );
            return;
        }


        boolean passInt = true;
        Class[] fparms = new Class[1];
        fparms[0] = java.lang.Integer.TYPE;
        Constructor cn = null;
        try { cn = UI.getConstructor( fparms ); }
        catch (NoSuchMethodException e) { passInt = false; }
        
        Object[] parms = new Object[1];
        try {
            Object iface;
            parms[0] = new Integer(where);;
            if (passInt)
                iface = cn.newInstance( parms );
            else
                iface = UI.newInstance();
            Thread.sleep( 500 );
            doGame( (NebulaUi)iface );
			com.end(); com = null;
            syncSock.close();
            return;
        }
        catch (Exception e) {
            System.err.println( "Unexpected Error in client: " + e.getMessage() );
			e.printStackTrace();
            try { syncSock.close(); }
            catch (IOException f) {}
            return;
        }
    }

    void doGame( NebulaUi iface )
        throws Exception
    {
        boolean ourBid = false, ourPlay = false, ourPass = false;
        int[] scores, bags = new int[4];
        int who;
        PlayingCard c = new PlayingCard();
        
        syncSock.setSoTimeout( 0 );
        boolean alive = true;
		int delay = 0;
        while (alive) {
			if (packets.empty())
				block( delay );
			// linearly increasing idle period.
			// for "fast" computer players, this value will stay low.
			// delay is reset to zero when there is activity from the user.
			delay = (delay > 1100) ? 1100 : (delay + 15);

            if (!packets.empty()) {
				Object o = packets.dequeue(); // try to read data
				//System.out.println ("client: "+o);
				if (o instanceof ChatPkt)
				{ iface.chat( ((ChatPkt)o).getMessage() ); }
				else if (o instanceof YourTurnPkt)
				{
					if (((YourTurnPkt)o).isBid())
					{
						iface.yourTurnToBid();
						ourBid = true;
					}
					else
					{
						iface.yourTurn();
						ourPlay = true;
					}
				}
				else if (o instanceof PassNotifyPkt)
				{
					iface.yourTurnToPass( ((PassNotifyPkt)o).getHowMany(),
							((PassNotifyPkt)o).getWho() );
					ourPass = true;
				}
				else if (o instanceof Player[])
				{ iface.setPlayers( (Player[])o ); }
				else if (o instanceof DealPkt)
				{ iface.dealHand( ((DealPkt)o).getCards() ); }
				else if (o instanceof AcceptPkt)
				{ iface.accepted(); }
				else if (o instanceof RejectPkt)
				{ iface.rejected(); }
				else if (o instanceof TrumpPkt)
				{
					iface.setTrump(
						((TrumpPkt)o).getTrump(),
						((TrumpPkt)o).getTrumpName() );
				}
				else if (o instanceof GameNamePkt)
				{ iface.setGameName( ((GameNamePkt)o).getName() ); }
				else if (o instanceof GameResult)
				{ iface.endGame( (GameResult)o ); }
				else if (o instanceof TrickPkt)
				{ iface.clearTableau( ((TrickPkt)o).getWho() ); }
				else if (o instanceof PlayPkt)
				{
					iface.cardToTableau( ((PlayPkt)o).getWho(),
							((PlayPkt)o).getCard() );
				}
				else if (o instanceof BidPkt)
				{
					BidPkt bp = (BidPkt)o;
					iface.setBid( bp.getWho(), bp.getBid() );
				}
				else if (o instanceof SofarPkt)
				{ iface.playedSoFar( ((SofarPkt)o).getCards() ); }
				else if (o instanceof BootPkt)
				{
					inGame = false;
					iface.booted( ((BootPkt)o).getMessage() );
					return;
				}
				else if (o instanceof EndHandPkt)
				{ iface.endHand(); }
				else if (o instanceof QueryPkt)
				{ 
					com.send( new ResponsePkt(
								iface.respond(((QueryPkt)o).toString())) );
				}
				else // unrecognized packets get sent to miscInfo
				{
					iface.miscInfo( o );
				}
            }

            // incoming packets have been checked, so now we
            // send any packets we need to send.
            if (ourBid && iface.ready()) {
				com.send( new BidPkt(iface.getBid(), 0) );
				ourBid = false;
				delay = 0;
            }
            if (ourPlay && iface.ready()) {
				com.send( new PlayPkt(iface.getPlay(), 0) );
                ourPlay = false;
				delay = 0;
            }
			if (ourPass && iface.ready())
			{
				com.send( new PassPkt(iface.getPass()) );
				ourPass = false;
				delay = 0;
			}
            if (iface.wantToQuit()) {
				com.send( new LeavePkt() );
                inGame = false;
                return;
            }
            if (iface.chatReady())
			{
				com.send( new ChatPkt(iface.getChat()) );
				delay = 0;
			}
        }
		com.end();
        return;
    }

	private synchronized void block( long l )
	{
		try { wait( l ); }
		catch (InterruptedException e) {}
	}
	private synchronized void unblock() { notifyAll(); }

	public void packetReceived( int id, Object o )
	{
		if (id != 0) return;
		packets.enqueue( o );
		unblock();
	}
}
