package net.sf.nebulacards.main;

/**
 * Interface to be implemented by all rule sets that require the trump to be
 * set by asking a player.  After bidding and passing has taken place,
 * the server asks whom it should ask for the trump,
 * then it reports the player's response and asks for confirmation.  The
 * process is repeated until the confirmation succeeds.
 * At that point, a call is made to getTrump() to retrieve the trump.
 * @author James Ranson
 */
public interface AskTrumpRules
{
	/**
	 * Tell the server whom to ask for trump.
	 * @return The player who gets to pick trump.
	 */
	int whoPicksTrump();
	
	/**
	 * Read and parse the player's response.
	 * @param s The response the user gave.
	 * @return <i>true</i> if and only if the response represents a valid
	 * choice of trump.
	 */
	boolean trumpResponse( String s );
}
