// July 1999

package net.sf.nebulacards.main;

/**
 * Set of methods that must be implemented by any rule set that requires
 * bidding.
 *
 * @author James Ranson
 */
public interface BiddingRules
{
	/**
	 * Tell server if a certain player should bid.
	 * During the bidding process, if applicable, the bidding will rotate
	 * clockwise around the table.  Before the server queries the player to
	 * bid, this method will be called.  If it returns true, the player will
	 * be queried, otherwise he will be skipped.  Bidding stops upon
	 * receiving false from this method on four consecutive calls.
	 *
	 * @return True if player should be queried, false if he should be skipped.
	 */
	boolean shouldBid( int position );

	/**
	 * Called by server on completion of the bidding process.
	 * This is a courtesy to rule set programmers.
	 */
	void doneBidding();

	/**
	 * Receive information about a player's bid.
	 * This method is called immediately after a player has responded to a
	 * bid query.
	 * @param pos Position number of bidding player.
	 * @param bid The player's bid.
	 */
	void reportBid( int pos, int bid );
}

