// July 1999
package net.sf.nebulacards.main;

import java.io.*;

/**
 * A simple class to represent the result of a card game.
 *
 * @author James Ranson
 */
public class GameResult
    implements Externalizable
{
    /**
     * Flag indicating whether the game is over or not.
     * You must set this yourself, there are no methods to set it.
     */
    public boolean done;

    /**
     * The array containing the numeric value of each winner.
     *
     * @see #setWinners
     */
    public int[] winners;

    /**
     * Sets the array of winners
     *
     * @see #winners
     * @param w The array of winners to be copied into this object.
     */
    public void setWinners( int[] w )
    {
        winners = (int[])w.clone();
    }

    /**
     * Generates a string representation of this object
     */
    public String toString()
    {
        String s;
        if (!done)
            s = "Game is not complete.";
        else {
            s = "Game is complete.  Winners are: ";
            for (int i = 0; i < winners.length; i++)
                s += String.valueOf( winners[i] ) + " ";
        }
        return s;
    }

    /**
     * Make a copy of this object
     *
     * @return A new GameResult object that is the same as this one.
     */
    public GameResult copy()
    {
        GameResult g = new GameResult();
        g.done = done;
        if (done)
            g.setWinners( winners );
        return g;
    }

    /**
     * Output serialization
     * You do not need to call this method.  It is called by
     * java.io methods when this object is to be written to a stream.
     */
    public void writeExternal(ObjectOutput out)
        throws IOException
    {
        out.writeBoolean( done );
        if (done)
            out.writeObject( winners );
    }

    /**
     * Input deserialization
     * You do not need to call this method.  It is called by
     * java.io methods when this object is to be read from a stream.
     */
    public void readExternal(ObjectInput in)
        throws IOException
    {
        done = in.readBoolean();
        if (done) {
            try { winners = (int[])in.readObject(); }
            catch (Exception e) {}
        }
    }
}
