// January 2001

package net.sf.nebulacards.main;

/**
 * Rules for games that involve passing.  This interface must be implemented by any rule set that
 * involves passing.  (Such a rule class must still be a subclass of Rules.)  The server checks if
 * its rule set implements this interface, and if so, the server assumes the game requires passing.
 *
 * The passing process works as follows:
 * numPass() is called for each player.  If it returns non-zero, then wherePass() is called.
 * The server waits for all passes to come in, then processes them.
 * This process is repeated until numPass() returns zero for every player.
 *
 * @author James Ranson
 */
public interface PassingRules
{
	/**
	 * Tell the server how many cards a player should pass.
	 *
	 * @param p The table index of the player in question.
	 * @return Number of cards the player should pass.
	 */
	int numPass( int p );

	/**
	 * Tell the server where a player's passed cards should go.
	 *
	 * @param p The table index of the player in question.
	 * @return Table index of the player who should receive the passed cards.
	 */
	int wherePass( int p );
}
