package net.sf.nebulacards.main;

/**
 * A card player.
 * @author James Ranson
 */
public class Player
	implements java.io.Serializable, Cloneable
{
	private String name;
	private int score, bags, bid;
	private boolean hasBid;

	/**
	 * Construct a new player.
	 */
	public Player()
	{
		name = null;
		score = bags = bid = 0;
	}

	/**
	 * Construct a new player.
	 *
	 * @param n The player's name.
	 */
	public Player( String n )
	{
		name = n;
		score = bags = bid = 0;
	}

	/**
	 * Get the player's name.
	 */
	public synchronized String getName() { return name; }
	
	/**
	 * Get the player's score.
	 */
	public synchronized int getScore() { return score; }
	
	/**
	 * Get the player's bags.
	 */
	public synchronized int getBags() { return bags; }

	/**
	 * Get the player's bid.
	 */
	public synchronized int getBid() { return bid; }

	/**
	 * Set the player's name.
	 * @param n The new name.
	 */
	public synchronized void setName( String n ) { name = n; }
	
	/**
	 * Set the player's score.
	 * @param n The new score.
	 */
	public synchronized void setScore( int n ) { score = n; }
	
	/**
	 * Set the player's bags.
	 * @param n The new bags.
	 */
	public synchronized void setBags( int n ) { bags = n; }
	
	/**
	 * Set the player's bid.
	 * @param n The new bid.
	 */
	public synchronized void setBid( int n )
	{
		bid = n;
		hasBid = true;
	}

	/**
	 * Clear the player's bid.  This will cause hasBid() to return false
	 * until setBid(int) is called again.
	 * @see setBid(int)
	 * @see hasBid()
	 */
	public synchronized void clearBid()
	{
		bid = -1;
		hasBid = false;
	}

	/**
	 * Determine if the player has registered a bid.
	 * @return True if player has bid, false otherwise.
	 */
	public synchronized boolean hasBid()
	{
		return hasBid;
	}

	/**
	 * Set the player's score and bags.
	 * @param s The new score.
	 * @param b The new bags.
	 */
	public void setScoreAndBags( int s, int b )
	{
		setScore( s ); setBags( b );
	}

	/**
	 * Add to the player's current score.
	 * @param sdiff The amount to add (can be negative).
	 */
	public void incScore( int sdiff ) { score += sdiff; }
	/**
	 * Add to the player's current bags.
	 * @param bdiff The amount to add (can be negative).
	 */
	public void incBags( int bdiff ) { bags += bdiff; }
	public String toString()
	{
		String s = "Name: " + getName() + " ; Score=" + getScore() + " ; ";
		s += "Bags=" + getBags() + " ; Bid=" + getBid();
		return s;
	}
}
