package net.sf.nebulacards.main;

/**
 * Queue.
 * Java 1.1 seems to be without a Queue class (it has a Stack!).
 * @author James Ranson
 */
import java.util.*;

public class Queue
	extends Vector
{
	/**
	 * Put an item at the back of the queue.
	 * @param o The item to be put in the queue.
	 */
	public synchronized void enqueue( Object o )
	{
		super.addElement( o );
	}

	/**
	 * Take an item from the front of the queue.
	 * @return The first item on the queue, or null if the queue is empty.
	 */
	public synchronized Object dequeue()
	{
		if (empty()) return null;
		Object o = super.firstElement();
		super.removeElementAt( 0 );
		return o;
	}

	/**
	 * Look at first item without removing it.
	 * @return The first item in the queue.
	 */
	public synchronized Object peek()
	{
		if (empty()) return null;
		return super.firstElement();
	}

	/**
	 * Test if the queue is empty.
	 * @return True if the queue does <b>not</b> have any entries, else false.
	 */
	public synchronized boolean empty()
	{
		return (super.size() < 1);
	}

	/**
	 * Remove all elements in this queue.
	 */
	public synchronized void removeAll()
	{
		super.removeAllElements();
	}
}
