// Rules module for Hearts
// James Ranson
// March 2001

package net.sf.nebulacards.rules;
import net.sf.nebulacards.main.*;

public class Hearts
    extends net.sf.nebulacards.main.Rules
	implements PassingRules
{
	protected int handCount;
	
    public Hearts()
	{
		handCount = 0;
		trump = NO_TRUMP;
		trumpName = "No Trump";
	}

    public String name() { return "Hearts"; }

    // check that card 'c' is a valid play in this situation
    public boolean validPlay( PileOfCards beenPlayed,
                              PileOfCards tableau,
                              PileOfCards hand,
                              PlayingCard c )
    {
        // if there is only one suit in the hand, then any play is legal
        if (hand.allThisSuit(c.getSuit())) return true;
        int led;
        if (tableau.howManyCards() == 0) led = -1;
        else led = tableau.getCard(0).getSuit();

        if (c.getSuit() != led && hand.anyOfThisSuit(led)) return false;
        // now we only need to check for leading hearts too soon
        if (led == -1  // if we led
            && (c.getSuit() == PlayingCard.HEARTS)  // and we played a heart 
            && !beenPlayed.anyOfThisSuit( PlayingCard.HEARTS )
           )
            return false;

        // if the card has survived this long, then it is valid
        return true;
    }
            
        

    // update the scores
    public void score( PileOfCards[] hands, Player[] p )
    {
		for (int i = 0; i < 4; i++)
		{
			int points = 0;
			points += hands[i].cardsInSuit(PlayingCard.HEARTS).howManyCards();
			if (hands[i].search( "Qs" ) != PileOfCards.NOT_FOUND)
				points += 13;
			// shot the moon?
			if (points == 26)
			{
				points = 0;
				for (int j = 0; j < 4; j++)
					if (j != i)
						p[j].setScore( p[j].getScore() + 26 );
				break;
			}
			p[i].setScore( p[i].getScore() + points );
		}
    }

    // furnish a result for this game, including whether the game is complete
    public GameResult done( Player[] p )
    {
        GameResult gr = new GameResult();
		int highest = 0;
		boolean tied = false;
		
		// This method is called precisely once per hand, so we
		// increment the hand counter here.
		handCount++;

		gr.winners = new int[1];
		for (int i = 0; i < 4; i++)
		{
			int score = p[i].getScore();
			if (score == highest)
				tied = true;
			else if (score > highest)
			{
				highest = score;
				gr.winners[0] = i;
			}
		}
		gr.done = (highest >= 100 && !tied);
        return gr;
    }

	// Passing Methods
	public int numPass( int p ) { return (handCount % 4 == 3) ? 0 : 3; }
	public int wherePass( int p )
	{
		// where to pass depends on hand count (left,right,across,hold)
		final int loc[][] = {
			{ 1, 3, 2, 0 }, // player 0
			{ 2, 0, 3, 1 }, // player 1
			{ 3, 1, 0, 2 }, // player 2
			{ 0, 2, 1, 3 }  // player 3
		};
		return loc[p][handCount % 4];
	}
}
