// Rules module for Spades
// James Ranson
// July 1999

package net.sf.nebulacards.rules;
import net.sf.nebulacards.main.*;

public class Spades
    extends net.sf.nebulacards.main.Rules
	implements BiddingRules
{
	protected int bidCount;

    public Spades()
	{
		trump = PlayingCard.SPADES;
		trumpName = "Spades";
		bidCount = 0;
	}
    public void setTrump( PlayingCard c ) {}

    // does this game involve bidding
    // public boolean bidding() { return true; }
	// Bidding
	public boolean shouldBid( int pos ) { return bidCount < 4; }
	public void doneBidding() { bidCount = 0; }
	public void reportBid( int pos, int bid ) { bidCount++; }

    public String name() { return "Spades"; }

    // check that card 'c' is a valid play in this situation
    public boolean validPlay( PileOfCards beenPlayed,
                              PileOfCards tableau,
                              PileOfCards hand,
                              PlayingCard c )
    {
        // if there is only one suit in the hand, then any play is legal
        if (hand.allThisSuit(c.getSuit())) return true;
        int led;
        if (tableau.howManyCards() == 0) led = -1;
        else led = tableau.getCard(0).getSuit();

        if (c.getSuit() != led && hand.anyOfThisSuit(led)) return false;
        // now we only need to check for leading spades too soon
        if (led == -1  // if we led
            && (c.getSuit() == PlayingCard.SPADES)  // and we played a spade
            && !beenPlayed.anyOfThisSuit( PlayingCard.SPADES )
           )
            return false;

        // if the card has survived this long, then it is valid
        return true;
    }
            
        

    // update the scores
    public void score( PileOfCards[] hands, Player[] p )
    {
        int i;
        int[] tricks = new int[4], scoreDiffs = { 0, 0 };

        // establish how many tricks each player won
        for (i = 0; i < 4; i++)
            tricks[i] = hands[i].howManyCards() / 4;
        
		// consider nil bids
        for (i = 0; i < 4; i++) {
            if (p[i].getBid() == 0) {
                if (tricks[i] == 0) scoreDiffs[i % 2] += 100;
                else scoreDiffs[i % 2] -= 100;
            }
        }
		// score the rest of the bids
        for (i = 0; i < 2; i++) {
            int bidDiff = tricks[i] + tricks[i + 2] - p[i].getBid()
							- p[i + 2].getBid();
            if (bidDiff < 0)  // set
                scoreDiffs[i] -= (p[i].getBid() + p[i + 2].getBid()) * 10;
            else {
                scoreDiffs[i] += (p[i].getBid() + p[i + 2].getBid()) * 10;
                scoreDiffs[i] += bidDiff;
                p[i].setBags( p[i].getBags() + bidDiff );
                p[i + 2].setBags( p[i + 2].getBags() + bidDiff );
            }
            while (p[i].getBags() >= 10) {
                scoreDiffs[i] -= 100;
                p[i].setBags( p[i].getBags() - 10 );
                p[i + 2].setBags( p[i + 2].getBags() - 10 );
            }
            p[i].setScore( p[i].getScore() + scoreDiffs[i] );
            p[i + 2].setScore( p[i + 2].getScore() + scoreDiffs[i] );
        }
    }

    // furnish a result for this game, including whether the game is complete
    public GameResult done( Player[] p )
    {
        GameResult gr = new GameResult();
        if ( p[0].getScore() >= 500 || p[1].getScore() >= 500 ) gr.done = true;
        else gr.done = false;
        if ( p[0].getScore() == p[1].getScore() ) gr.done = false; // no ties
		gr.winners = new int[2];
        if ( p[0].getScore() > p[1].getScore() )
		{
            gr.winners[0] = 0; gr.winners[1] = 2;
		}
        else
		{
			gr.winners[0] = 1; gr.winners[1] = 3;
		}
        return gr;
    }

}
