// Server for Nebula Cards
// James Ranson
// July 1999

package net.sf.nebulacards;

import java.net.*;
import java.io.*;
import net.sf.nebulacards.main.*;
import net.sf.nebulacards.rules.*;

class stfront
{
    public static void main( String[] args )
        throws Exception
    {
        BufferedReader in = new BufferedReader( new InputStreamReader(System.in) );
        String rulesString = null;
        String portString = null;

        // parse command line
        for (int i = 0; i < args.length; i++) {
            if (args[i].equals( "-p" )) {
                try { portString = args[++i]; }
                catch (Exception e) { portString = null; }
                continue;
            }
            if (args[i].equals( "-r" )) {
                try { rulesString = args[++i]; }
                catch (Exception e) { rulesString = null; }
                continue;
            }
        }
        
        // open rules class
        if (rulesString == null) {
            System.err.println( "Please specify a rule set [Spades]: " );
            try { rulesString = in.readLine(); }
            catch (IOException e) {}
            if (rulesString.equals(""))
                rulesString = "Spades";
			if (rulesString.indexOf( '.' ) < 0)
				rulesString = "net.sf.nebulacards.rules." + rulesString;
        }
        Class ruleSet;
        try { ruleSet = Class.forName(rulesString); }
        catch (Exception e) {
            System.err.println( "Can't open rules: " + rulesString );
            try { Thread.sleep( 4000 ); }
            catch (Exception f) {}
            return;
        }

        // bind socket
        int port;
        if (portString == null) {
            System.err.print( "Listening port [8802]: " );
            System.err.flush(); String s = null;
            try { s = in.readLine(); }
            catch (IOException e) {}
            if (s.equals(""))
                port = 8802;
            else
                port = Integer.parseInt( s );
        }
        else port = Integer.parseInt( portString );

        serv server = new serv( (Rules)ruleSet.newInstance(), port );
        server.start();
        System.err.println( "OK, the server was started on port " + port + "." );
        System.err.println( "You now have the opportunity to load clients." );
        System.err.println( "The game needs 4 clients to start; they can be loaded" );
        System.err.println( "from this program, as separate programs, or even on" );
        System.err.println( "other computers.  Loading clients from this program" );
        System.err.println( "will improve the speed of the game.\n\n" );

        client[] comps = new client[4];
        getClients( comps, server, port );
        System.err.println( "Game is over.  Waiting for clients to shut down." );
        System.err.println( "If this takes more than ten seconds, just close this window." );
        boolean flag = true;
        while (flag) {
            Thread.yield();
            flag = false;
            for (int i = 0; i < 4; i++)
                if (comps[i] != null && comps[i].isAlive())
                    flag = true;
        }
        System.exit( 0 );
    }

	public static void prompt()
	{
		System.err.println( "1) Start a human interface." );
		System.err.println( "2) Start a Spades computer." );
        System.err.println( "3) Start another client." );
        System.err.println( "4) Boot a player from the game." );
	}

    public static void getClients( client[] c, serv server, int port )
        throws IOException
    {
        BufferedReader in = new BufferedReader( new InputStreamReader(System.in) );
        String s = null, cname = null;
		prompt();
        while ( server.isAlive() ) {
            String ins = null;
            try { ins = in.readLine(); }
            catch (IOException e) { return; }
			// If we get an EOF, then the user doesn't want to enter any more
			// commands, so we can just join the server thread.
			if (ins == null)
			{
				try { server.join(); }
				catch (InterruptedException e) {e.printStackTrace();}
				break;
			}
			if (ins.equals("1"))
			{
				if (server.howMany() == 4)
					System.err.println("No room for another player.");
				else startClient( in, c, "net.sf.nebulacards.ui.gui",
									server, port );
			}
			if (ins.equals("2"))
			{
				if (server.howMany() == 4)
					System.err.println("No room for another player.");
				else startClient( in, c, "net.sf.nebulacards.ui.SpadesComp",
									server, port );
			}
            if (ins.equals("3"))
			{
                if (server.howMany() == 4)
                    System.err.println("The game already has enough players.");
                else startClient( in, c, null, server, port );
            }
            if (ins.equals("4"))
			{
                if (server.howMany() == 0)
                    System.err.println( "The game does not have any players." );
                else bootClient( in, server );
            }
			prompt();
        }
    }

    public static void startClient( BufferedReader in,
                                    client[] c,
									String ifname,
                                    serv server, int port )
    {
        String cname = null;
        System.err.print( "Player's name? " );
        System.err.flush();
        try {
	        cname = in.readLine();
			if (ifname == null)
			{
    	        System.err.print( "Which Client?: " );
        	    System.err.flush();
            	ifname = in.readLine();
			}
        }
        catch (IOException e) { return; }
        String[] parms = {
            "-p", String.valueOf(port),
            "-h", "127.0.0.1",
            "-n", cname,
            "-i", new String( ifname )
        };
        int w = -1;
        for (int i = 0; i < 4; i++)
            if (c[i] == null || !c[i].isAlive())
                w = i;
		if (w < 0) return;

        c[w] = new client( parms );
        c[w].start();
        System.err.println( "Client started." );
    }

    public static void bootClient( BufferedReader in, serv server )
        throws IOException
    {
        String[] n = server.getNames();
        for (int i = 0; i < 4; i++) {
            System.err.print( i );
            System.err.println( " - " + n[i] );
        }
        System.err.print( "Boot which player (blank to cancel)? " );
        System.err.flush();
        String s = in.readLine();
		if (s.equals("")) return;
        int who = -1;
        for (int i = 0; i < n.length; i++)
            if (s.equalsIgnoreCase(n[i])) { who = i; break; }
        if (who == -1) {
            try { who = Integer.parseInt( s ); }
            catch (NumberFormatException e) { who = -1; }
        }
        if (who < 0 || who > 3 || n[who] == null ) {
            System.err.println( "cancelled." );
            return;
        }
        System.err.println( "Why do you want to boot " + n[who] + "?" );
        s = in.readLine();
        server.tellBoot( who, s );
    }
}
