// Computer player for Spades
// James Ranson
// August 1999

package net.sf.nebulacards.ui;

import net.sf.nebulacards.main.*;
import net.sf.nebulacards.rules.*;

public class SpadesComp
	extends net.sf.nebulacards.uicomponents.ComputerStub
{
    private boolean verbose = false;
    protected int partner;
    
    // constructor
    public SpadesComp( int w )
    {
		super( w );
        partner = (w + 2) % 4;
    }


    /* Thinking methods */

    protected int thinkBid()
    {
        int tally = 0;
        for (int i = 0; i < hand.howManyCards(); i++)
            if (hand.getCard( i ).getValue() > 11
                || hand.getCard( i ).getValue() == 1
               )
                tally++;
        if (tally > 0)
            return tally;
        // some final checks before bidding nil
        if ( hand.cardsInSuit( PlayingCard.SPADES ).howManyCards() > 4 )
            tally = 1;
        if ( hasBid[partner] && bids[partner] == 0 )
            tally = 2;
        if (verbose) System.out.println( "Bid: " + String.valueOf(tally) );
        return tally;
    }

    protected PlayingCard thinkPlay()
    {
        boolean pWinning, pNil, meNil, pSafe;
        Spades rules = new Spades();
		PlayingCard myPlay = null;

        pNil = (bids[partner] == 0); meNil = (bids[whereAmI] == 0);
        pWinning = ( tableau.howManyCards() > 1
                     && rules.whoWinsTrick(tableau) == partner );
        pSafe = (tableau.hasPlayed(partner) && !pWinning) || !pNil;

        boolean madeBid = ( tricks[partner] + tricks[whereAmI] >= bids[partner] + bids[whereAmI] );
        int wantToWin = 0;   // + means we want to win the trick, - means no
        if (pNil && !pSafe) wantToWin += 20;
        if (meNil && tricks[whereAmI] == 0) wantToWin -= 20;
        if (madeBid)
            wantToWin -= 5;
        else wantToWin += 5;
        switch (tableau.howManyCards())
        {
        case 3: // we play the last card.
            if (pWinning) wantToWin -= 6;
            break;
        case 2: // one opponent gets to play after us
            if (pWinning &&
                Think.canBeat( otherHands[(whereAmI+1) % 4], tableau, beenPlayed, rules ) < 0.35
               ) wantToWin -= 6;
            break;
        }

        PileOfCards vp = Think.validPlays( hand, tableau, beenPlayed, rules );
        PileOfCards tl = Think.takeLead( vp, tableau, rules );

        if (verbose) {
            System.out.print( "Want to Win: " ); System.out.println( wantToWin );
            System.out.println( "Tableau: " + tableau );
            System.out.println( "Valid Plays: " + vp );
            System.out.println( "Take Lead: " + tl );
        }

        if (wantToWin >= 0) {
            if (tl.howManyCards() > 0) {  // want to win and can
                if (tableau.howManyCards() == 3)
                    myPlay = Think.lowCard( tl, true );
                else
                    myPlay = Think.highCard( tl, true );
            }
            else  // want to win but can't
                myPlay = Think.lowCard( vp, true );
        }
        else { // don't want to win
            if (tl.howManyCards() < vp.howManyCards()) { // don't have to win
                vp.remove( tl );  // disregard winning cards
                if (pNil || (pWinning && !meNil) ) myPlay = Think.lowCard( vp, true );
                else myPlay = Think.highCard( vp, true );
            }
            else {  // have to take lead
                if (tableau.howManyCards() == 3) // guaranteed to win
                    myPlay = Think.highCard( vp, true );
                else { // possibility of rescue
                    // avoid trumping
                    if (!vp.allThisSuit( PlayingCard.SPADES ))
                        vp.removeSuit( PlayingCard.SPADES );
                    myPlay = Think.lowCard( vp, true );
                }
            }
        }
                

        if (verbose) {
            System.out.println( "Choice: " + myPlay );
            System.out.println();
        }
		return myPlay;
    }


    public void cardToTableau( int pos, PlayingCard c )
    {
		// All the processing done in the stub still applies.
		super.cardToTableau( pos, c );

		// However, we can make stronger statements about what cards
		// the other players cannot have. In particular, if the card
		// is sluffed, then that player cannot have any cards
        // from the lead suit
        if (pos != whereAmI
            && pos != tableau.getLead()
            && c.getSuit() != tableau.getCard( 0 ).getSuit()
           )
            otherHands[pos].removeSuit( tableau.getCard(0).getSuit() );
    }
    
    public int getBid() { return thinkBid(); }

	public PileOfCards getPass() { return null; }

    // we must provide these functions, but they are not useful to us
	public String respond( String q ) { return ""; }
	public void booted( String why ) {}
	/*
	public void setGameName( String n ) {}
    public void chat( String s ) {}
    public boolean chatReady() { return false; }
    public String getChat() { return null; }
    public void endGame( GameResult gr )
    {
        if (verbose) System.out.println( gr );
        quitNow = true;
    }
    public boolean wantToQuit() { return quitNow; }
    
    public void yourTurnToBid() { if (verbose) System.out.println( "My turn to bid" ); }
    public void yourTurn() { if (verbose) System.out.println( "My turn to play." ); }
	public void yourTurnToPass(int howmany, int who ) {}

    public boolean ready() { return true; }
	*/
    public void accepted() {} // we take acceptance for granted
	
    public void rejected()
    {
        System.err.println( "Panic: play was rejected.  Something is wrong!" );
    }
}
