// AWT user interface for Nebula Cards

package net.sf.nebulacards.ui;

import java.awt.*;
import java.awt.event.*;
import net.sf.nebulacards.main.*;
import net.sf.nebulacards.uicomponents.*;

public class gui extends Frame
    implements ActionListener, WindowListener, NebulaUi, CardListener
{
    PileOfCards hand, pass;
	BasicDataBox[] dboxes = new BasicDataBox[4];
    int[] tricks = { 0, 0, 0, 0 };
	BasicCardSelector cards;
    SimpleTableTop tableau;
    Label trumpLabel;
    boolean quitNow = false, cardReady = false, chatReady = false;
    boolean myTurn = false, myBidTurn = false;
    int myBid;
    PlayingCard myPlay;
    int playIndex;
    Panel cardsPanel;
    TextArea msgBox;
    TextField chatBox;
    String chatString = "";
	int whereAmI;

    public gui (int w)
    {
		whereAmI = w;
        // we always want to be sitting at the bottom of the screen, so
        // we have to adjust the position of the players around OUR table.
        // Players keep the same table index, but NORTH may no longer be
        // at the top of the table.
        final int[][] positions = {
            { 2, 3, 0, 1 },
            { 3, 0, 1, 2 },
            { 0, 1, 2, 3 },
            { 1, 2, 3, 0 }
        };
        
        GridBagLayout grid = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();

        // set up the window
		setBackground( Color.white );
        addWindowListener( this );
        setLayout( grid );

        for (int j = 0; j < 4; j++) 
			dboxes[j] = new BasicDataBox( Color.black, Color.white );

        // construct tableau panel
		tableau = new SimpleTableTop( w, Color.yellow, Color.blue );

        Panel infoPanel = new Panel();
        trumpLabel = new Label( "Trump:           " );
        infoPanel.add( trumpLabel );
		
        GridBagLayout boxl = new GridBagLayout();
        Panel boxes = new Panel( boxl );
        gbc.fill = gbc.BOTH; gbc.gridy = gbc.RELATIVE; gbc.gridx = 0;
        gbc.gridwidth = gbc.REMAINDER; gbc.gridheight = 1;
        chatBox = new TextField( 55 );
		chatBox.setBackground( Color.white );
        chatBox.addActionListener( this );
        boxl.setConstraints( chatBox, gbc );
        boxes.add( chatBox );
        msgBox = new TextArea( "Welcome to Nebula Cards by James Ranson.\n", 5, 55 );
        msgBox.setEditable( false );
		msgBox.setBackground( Color.white );
        gbc.gridheight = gbc.REMAINDER;
        boxl.setConstraints( msgBox, gbc );
        boxes.add( msgBox );
        gbc = new GridBagConstraints();

        /***************************************/
        // now, add all components to the window
        Panel p;
        gbc.fill = gbc.BOTH;
        gbc.gridx = 0; gbc.gridy = 0;
        gbc.gridheight = 3; gbc.gridwidth = 4;
        grid.setConstraints( infoPanel, gbc ); add( infoPanel );
        gbc.gridwidth = 6; gbc.gridx = gbc.RELATIVE;
        grid.setConstraints( dboxes[positions[w][NORTH]], gbc );
		add ( dboxes[positions[w][NORTH]] );
        gbc.gridwidth = gbc.REMAINDER;
        p = new Panel(); grid.setConstraints( p, gbc ); add( p );

        gbc.weightx = 1.0;
        gbc.gridx = 0; gbc.gridy = gbc.RELATIVE; gbc.gridwidth = 4;
        grid.setConstraints( dboxes[positions[w][WEST]], gbc );
		add( dboxes[positions[w][WEST]] );
        gbc.gridx = gbc.RELATIVE; gbc.gridwidth = 5;
        grid.setConstraints( tableau, gbc ); add( tableau );
        gbc.gridwidth = 4;
        grid.setConstraints( dboxes[positions[w][EAST]], gbc );
		add( dboxes[positions[w][EAST]] );
        gbc.weightx = 0;

        gbc.gridx = 0; gbc.gridwidth = 4; gbc.gridheight = 3;
        gbc.gridy = 6;
        p = new Panel(); grid.setConstraints( p, gbc ); add( p );
        gbc.gridwidth = 6; gbc.gridx = gbc.RELATIVE;
        grid.setConstraints( dboxes[positions[w][SOUTH]], gbc );
		add ( dboxes[positions[w][SOUTH]] );
        gbc.gridwidth = gbc.REMAINDER;
        p = new Panel(); grid.setConstraints( p, gbc ); add( p );

		// add card selector
		cards = new BasicCardSelector( 20 );
		cards.addListener( this );
        gbc.gridx = 0; gbc.gridy = 9;
        gbc.gridwidth = gbc.REMAINDER; gbc.gridheight = 2;
        grid.setConstraints( cards, gbc ); add( cards );

        gbc.gridx = 0; gbc.gridy = 11;
        gbc.gridwidth = gbc.REMAINDER; gbc.gridheight = gbc.REMAINDER;
        grid.setConstraints( boxes, gbc ); add( boxes );
        
        pack();
		setVisible( true );
        try { Thread.sleep( 2000 ); }
        catch (Exception e) {}
    }


    /****** NebulaUi functions ******/

	public void setPlayers( Player[] p )
	{
		for (int i = 0; i < 4; i++)
		{
			dboxes[i].setName( p[i].getName() );
            dboxes[i].setScore( p[i].getScore() );
            dboxes[i].setBags( p[i].getBags() );
            dboxes[i].setBid( p[i].getBid() );
		}
	}
			
	public void setGameName( String n )
	{
        setTitle( "Nebula Cards: " + n );
	}
    
	public void endHand()
	{
		for (int i = 0; i < 4; i++)
		{
			dboxes[i].setBid( -1 );
			tricks[i] = 0;
			dboxes[i].setTricks( 0 );
		}
	}
    
    public void setBid( int who, int bid )
    {
        dboxes[who].setBid( bid );
    }
    
    public void cardToTableau( int pos, PlayingCard c )
    {
        tableau.putCard( pos, c );
    }
    
    public void dealHand( PileOfCards h )
    {
        hand = h;
		hand.sort();
		cards.setHand( h );
    }
    
    public void clearTableau( int whoWonTrick )
    {
		dboxes[whoWonTrick].wonTrick( true );
        try { Thread.sleep( 3000 ); }
        catch (InterruptedException e) {}
		tableau.sweep();
        tricks[whoWonTrick]++;
		dboxes[whoWonTrick].setTricks( tricks[whoWonTrick] );
		dboxes[whoWonTrick].wonTrick( false );
    }

    public boolean wantToQuit() { return quitNow; }

    public void endGame( GameResult gr )
	{
		msgBox.append( "Game Over." );
    }

    public void accepted()
    {
        myTurn = false;
		cards.removeCard( myPlay );
        hand.remove( myPlay );
    }

    public void rejected()
    {
		new AlertWindow( this, "Illegal Play", "You can't play that card." );
    }

    public boolean ready() { return cardReady; }

    public int getBid() { cardReady = false; return myBid; }

    public boolean chatReady() {
        if (!chatReady)
            return false;
        chatString = chatString.trim();
        if (chatString.equals( "" ))
            chatReady = false;
        return chatReady;
    }
    
    public String getChat() { chatReady = false; return (chatString + "\n"); }

    public void chat( String s ) { msgBox.append( s ); }

    public void yourTurnToBid()
	{
		cardReady = false;
		myBidTurn = true;
		while (!cardReady)
		{
			QueryWindow bidwin = new QueryWindow( this, "Please enter your bid" );
			while (!cardReady)
			{
				String s = bidwin.waitForAnswer( 5000 );
				if (s != null)
				{
					try
					{
						myBid = Integer.parseInt( s );
						cardReady = true;
					}
					catch (Exception e) {}
					break;
				}
			}
		}
	}

    public void yourTurn() { dboxes[whereAmI].setTurn(true); Thread.yield(); myTurn = true; }

	public void yourTurnToPass( int howmany, int who )
	{
		cardReady = false;
		PassWindow pw = new PassWindow( this, hand, howmany,
				((who >= 0 && who < dboxes.length) ? dboxes[who].getName()
													: "nobody") );
		while ((pass = pw.waitForAnswer( 100000 )) == null);
		cardReady = true;
	}

    public PlayingCard getPlay() {
        cardReady = false;
        dboxes[whereAmI].setTurn( false );
        myTurn = false;
        Thread.yield();
        return myPlay;
    }

	public PileOfCards getPass()
	{
		cardReady = false;
		return pass;
	}

    public void setTrump( int t, String n )
    {
        trumpLabel.setText( "Trump: " + n );
    }

	public String respond( String q )
	{
		QueryWindow qw = new QueryWindow( this, q );
		String s = null;
		while ((s = qw.waitForAnswer(5000)) == null);
		return s;
	}

    public void booted( String why )
    {
        msgBox.append( "\nYou were booted because: " + why );
        try { Thread.sleep( 8000 ); }
        catch (InterruptedException e) {}
        dispose();
    }

	// We must implement this, but we'll just ignore the data.
	public void playedSoFar( PileOfCards bp )
	{}

	// Any miscellaneous info will get printed to the chat box.
	// Hopefully, the object that is sent has a meaningful toString().
	public void miscInfo( Object o )
	{
		msgBox.append( "*** " + o.toString() + " ***" );
	}
    

    //********** Event handlers ********/
    public void windowClosing( WindowEvent e ) { quitNow = true; dispose(); }
    public void windowOpened( WindowEvent e ) {}
    public void windowIconified( WindowEvent e ) {}
    public void windowDeiconified( WindowEvent e ) {}
    public void windowClosed( WindowEvent e ) {}
    public void windowActivated( WindowEvent e ) {}
    public void windowDeactivated( WindowEvent e ) {}

    public void actionPerformed( ActionEvent ev )
    {
        String e = ev.getActionCommand();
		Object source = ev.getSource();
        if ( source == chatBox ) {
            chatString = chatBox.getText();
            chatReady = true;
            chatBox.setText( "" );
        }
    }

	/**
	 * Wait for cardSelected events from the CardSelector.
	 * @param c The card that was chosen.
	 */
	public synchronized void cardSelected( PlayingCard c )
	{
		if ( myTurn ) {
            playIndex = hand.search( c );
            if (playIndex != PileOfCards.NOT_FOUND) {
                myPlay = c;
                cardReady = true;
            }
        }
	}	

    public static void main( String[] args )
        throws InterruptedException
    {
        gui j = new gui(0);
        while (!j.wantToQuit()) Thread.yield();
        System.exit( 0 );
    }
}
