package net.sf.nebulacards.uicomponents;

import net.sf.nebulacards.main.*;
import net.sf.nebulacards.uicomponents.*;
import java.awt.*;
import java.awt.event.*;


/* Dialog for passing cards */
public class PassWindow
	extends Dialog
	implements ActionListener, ItemListener
{
	protected Checkbox[] checks;
	protected Button okButton;
	protected int howmany, checked;
	protected PileOfCards hand;
	protected PileOfCards selection;
	protected boolean ready;

	public PassWindow( Frame parent, PileOfCards _hand, int h, String who )
	{
		super( parent, "Pass Cards", true );
		howmany = h; selection = null; hand = _hand;
		setLayout( new BorderLayout() );
		add( new Label("Select " + howmany + " cards to pass to " + who), "North" );
		Panel p = new Panel( new GridLayout(hand.howManyCards(), 1) );
		checks = new Checkbox[hand.howManyCards()];
		okButton = new Button( "O.K." );
		okButton.setEnabled( false );
		for (int i = 0; i < hand.howManyCards(); i++)
		{
			p.add( checks[i] = new Checkbox(
					hand.getCard(i).toString(), false) );
			checks[i].addItemListener( this );
		}
		checked = 0;
		add( p, "Center" );
		add( okButton, "South" );
		pack();
		okButton.addActionListener( this );
		show();
	}
	
	protected void deactivate()
	{
		selection = new PileOfCards( checked );
		okButton.removeActionListener( this );
		for (int i = 0; i < hand.howManyCards(); i++)
		{
			if (checks[i].getState())
				selection.addToTop( hand.getCard(i) );
			checks[i].removeItemListener( this );
		}
		ready = true;
		dispose();
	}

	/**
	 * Get the cards the user selected.
	 * @param timeout The maximum time to wait (in milliseconds).
	 * @return null if time elapsed without answer, else answer.
	 */
	public synchronized PileOfCards waitForAnswer( long timeout )
	{
		if (!ready)
		{
			try { wait( timeout ); }
			catch (InterruptedException e) {}
		}
		return selection;
	}
	
	public void itemStateChanged( ItemEvent e )
	{
		checked = 0;
		for (int i = 0; i < checks.length; i++)
			if (checks[i].getState())
				checked++;
		okButton.setEnabled( checked == howmany );
	}

	public void actionPerformed( ActionEvent e )
	{
		if (e.getSource() == okButton)
			deactivate();
	}


	/**
	 * Testing Framework
	 */
	public static void main( String[] args )
	{
		Frame parent = new Frame();
		PileOfCards deck = PileOfCards.getStandardDeck();
		deck.shuffle();
		PileOfCards hand = new PileOfCards( 13 );
		for (int i = 0; i < 13; i++)
			hand.addToTop( deck.takeTop() );
		PassWindow pw = new PassWindow( parent, hand, 3, "Bob" );
		PileOfCards selection;
		while ((selection = pw.waitForAnswer(500)) == null);
		System.out.println( selection );
		parent.dispose();
		System.exit( 0 );
	}
}
