package net.sf.nebulacards.uicomponents;
import java.awt.*;
import java.awt.event.*;
import net.sf.nebulacards.main.*;


/**
 * A very simple, AWT text-based, card table playing surface.
 * Suppports only four-player games.
 * @author James Ranson
 */
public class SimpleTableTop extends Panel
{
    protected Label[] tableau = new Label[4];

	/**
	 * Construct a table top with default colors.
	 * @param w The player's position at the table.
	 */
	public SimpleTableTop( int w )
	{
		construct( w, Color.yellow, Color.blue );
	}

	/**
	 * Construct a table top with the given colors.
	 * @param w The player's position at the table.
	 * @param fg The foreground color of the cards.
	 * @param bg The background color of the table.
	 */
	public SimpleTableTop( int w, Color fg, Color bg )
	{
		construct( w, fg, bg );
	}

	/**
	 * Put a card on the table.
	 * @param w The position of the player who played the card.
	 * @param c The card played.
	 */
	public void putCard( int w, PlayingCard c )
	{
		tableau[w].setText( c.toString() );
	}

	/**
	 * Remove a card from the table.
	 * @param w The position of the player who played the card.
	 */
	public void removeCard( int w )
	{
		tableau[w].setText( "" );
	}

	/**
	 * Remove all cards from the table.
	 */
	public void sweep()
	{
		for (int i = 0; i < 4; i++)
			tableau[i].setText( "" );
	}

	/**
	 * Do the real work of constructing a table top.
	 * Called exclusively by the real constructors.
	 * @param w The player's position at the table.
	 * @param fg The foreground color of the cards.
	 * @param bg The background color of the table.
	 */
    private final void construct( int w, Color fg, Color bg )
    {
        // we always want to be sitting at the bottom of the screen, so
        // we have to adjust the position of the players around OUR table.
        // Players keep the same table index, but NORTH may no longer be
        // at the top of the table.
        final int[][] positions = {
            { 2, 3, 0, 1 },
            { 3, 0, 1, 2 },
            { 0, 1, 2, 3 },
            { 1, 2, 3, 0 }
        };
        
        for (int j = 0; j < 4; j++) {
            int i = positions[w][j]; // correct for shift
            switch (i)
            {
            case NebulaUi.NORTH:
            case NebulaUi.SOUTH:
				tableau[j] = new Label( "CARD", Label.CENTER );
				break;
            case NebulaUi.WEST:
				tableau[j] = new Label( "CARD", Label.RIGHT );
				break;
            case NebulaUi.EAST:
				tableau[j] = new Label( "CARD", Label.LEFT );
				break;
            }
			tableau[j].setForeground( fg );
        }

        // construct tableau panel
        setLayout( new GridLayout( 3, 3 ) );
        add( new Panel() );
        add( tableau[ positions[w][NebulaUi.NORTH] ] );
        add( new Panel() );
        add( tableau[ positions[w][NebulaUi.WEST] ] );
        add( new Panel() );
        add( tableau[ positions[w][NebulaUi.EAST] ] );
        add( new Panel() );
        add( tableau[ positions[w][NebulaUi.SOUTH] ] );
        add( new Panel() );
        setBackground( bg );
	}
}
